require 'structure_mapper'
require 'securerandom'
require 'celluloid'
require 'celluloid/autostart'
require 'nestene/storage'
require 'nestene/actor/delayed_scheduler'
require 'nestene/actor/core'
require 'nestene/actor/auton_storage'
require 'nestene/actor/auton_queue'
require 'nestene/auton_context'
require 'nestene/callback'
require 'nestene/scheduled_method'
require 'nestene/executing_method'
require 'nestene/execution_error'
require 'nestene/executed_method'
require 'nestene/auton_execution_queue'
require 'nestene/auton_state'
require 'nestene/ui/app'

module Nestene

  class Registry

    def self.register_auton(type)
      @registered_autons ||= []
      @registered_autons << type unless @registered_autons.include?(type)
    end


    def self.registered_autons
      (@registered_autons || []).map(&:to_s)
    end

  end


  class Auton

    include StructureMapper::Hash

    def self.inherited(cls)
      Registry.register_auton(cls)
    end

  end

  class FooAuton < Auton

    def do_foo
    end

    attribute foo: String
  end

  class SelfValue
    def value
      self
    end
  end

  def self.class_from_string(name)
    name.split('::').inject(Object) do |mod, class_name|
      mod.const_get(class_name) or raise "Could not find class #{name}"
    end
  end

  def self.start_nestene(storage)
    Celluloid::Actor[:nestene_core] = Nestene::Actor::Core.new(storage)
    Celluloid::Actor[:delayed_scheduler] = Nestene::Actor::DelayedScheduler.new
    Celluloid::Actor[:auton_queue] = Nestene::Actor::AutonQueue.new
    storage.list.each do |auton_id|
      storage_actor=Nestene::Actor::AutonStorage.new(auton_id, storage)
      Celluloid::Actor["storage:%s" % auton_id] = storage_actor
      storage_actor.publish_initial_state
    end
  end

end

class Time
  def self.from_structure str
    str ? iso8601(str) : nil
  end

  def to_structure
    iso8601(3)
  end
end
