require 'json'

module Nestene


  class FileNameConverter

    def key_to_file_name(key)
      key.gsub(/[^a-zA-Z0-9]/){|x| "_%x" % x.ord}
    end

    def file_name_to_key file_name
      file_name.gsub(/_[a-zA-Z0-9]{2}/){|x| x[1..-1].hex.chr}
    end

  end

  class DiskStorage

    def initialize(dir)
      @dir=dir
      @converter = FileNameConverter.new
    end

    def list
      entries = Dir.entries(@dir)
      entries.delete('.')
      entries.delete('..')
      entries.map!{|fn| @converter.file_name_to_key(fn)}
      entries.delete('__credentials__')
      entries
    end

    def store(key, value)
      if value != nil
        final_name = "%s/%s" % [@dir,@converter.key_to_file_name(key)]
        tmp_name = "%s.temp" % final_name
        File.open(tmp_name,"w") do |f|
          f.write(JSON.pretty_generate(value))
        end
        File.rename(tmp_name, final_name)
      else
        delete(key)
      end
    end

    def load(key)
      file_name = "%s/%s" % [@dir,@converter.key_to_file_name(key)]
      if File.exist?(file_name)
        File.open(file_name,"r"){|f| JSON.parse(f.read)}
      else
        nil
      end
    end

    def delete(key)
      file_name = "%s/%s" % [@dir,@converter.key_to_file_name(key)]
      File.unlink(file_name)
    end

  end

  class MemoryStorage

    def initialize
      @storage = {}
    end

    def list
      @storage.keys
    end

    def load(key)
      @storage[key]
    end

    def store(key, value)
      if value != nil
        @storage[key] = value
      else
        delete(key)
      end
    end

    def delete(key)
      @storage.delete(key)
    end
  end

end
