require 'sinatra/base'
require 'haml'
require 'json'

module Nestene
  module Ui
    class App < Sinatra::Base


      get '/schedule' do
        content_type :json
        Celluloid::Actor[:delayed_scheduler].schedule_struct.to_json
      end

      get '/auton_types' do
        content_type :json
        Nestene::Registry.registered_autons.to_json
      end

      get '/autons.json' do
        content_type :json
        Celluloid::Actor[:nestene_core].auton_names.to_json
      end

      get '/credentials' do
        content_type :json
        Celluloid::Actor[:nestene_core].get_credentials.to_json
      end

      post '/credentials' do
        content_type :json
        begin
          credentials = JSON.parse(request.body.read)
          auton_id = Celluloid::Actor[:nestene_core].set_credentials credentials
          status 200
        rescue Exception => e
          puts e
          status 500
          {message: e.message}.to_json
        end
      end

      post '/autons' do
        content_type :json
        begin
          type = JSON.parse(request.body.read).fetch('auton_type')
          raise "auton_type not set" unless type
          auton_id = Celluloid::Actor[:nestene_core].create_auton type
          status 201
          {auton_id: auton_id}.to_json
        rescue Exception => e
          status 500
          {message: e.message}.to_json
        end

      end

      get '/' do
        @auton_names = Celluloid::Actor[:nestene_core].auton_names
        haml :index
      end

      get '/auton/create' do
        haml :auton_form
      end

      get '/auton/:id' do
        @auton_id = params[:id]
        @auton_state = Celluloid::Actor[:nestene_core].get_state(@auton_id)
        haml :auton
      end

      get '/auton/:id/schedule_step' do
        @auton_id = params[:id]
        @auton_state = Celluloid::Actor[:nestene_core].get_state(@auton_id)
        auton_type = @auton_state.type_as_class
        @available_methods = auton_type.instance_methods(false).map{|name| auton_type.instance_method(name)}
        @number_of_params = 0
        haml :schedule_step
      end

      post '/auton/:id/schedule_step' do
        parameters = params[:parameters] || []
        step_name = params[:step_name]

        @auton_id = params[:id]
        @auton_state = Celluloid::Actor[:nestene_core].get_state(@auton_id)
        auton_type = @auton_state.type_as_class
        @available_methods = auton_type.instance_methods(false).sort.map{|name| auton_type.instance_method(name)}

        method = @available_methods.find{|m| m.name.to_s == step_name}

        if method && method.arity == parameters.size
          step_id = Celluloid::Actor[:nestene_core].schedule_step(params[:id],step_name, parameters)
          headers['X-AUTON-STEP-ID'] = step_id
          redirect url('/auton/%s' % params[:id])
        else
          @method = method
          @number_of_params = method ? method.arity : 0
          @step_name = step_name
          haml :schedule_step
        end
      end

      post '/auton/create' do
        Celluloid::Actor[:nestene_core].create_auton params[:auton_type]
        redirect '/'
      end

    end

  end
end
